/*
    example.c :: RTC example
    Copyright (C) 2011 Collin Eggert

    RTC is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    RTC is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
    
    The following exception applies:  You may redistribute the documentation
    freely without attribution, but the accuracy of the information is not
    guarentied.
*/
#include <display_syscalls.h>
#include <keyboard_syscalls.h>
#include <CONVERT_syscalls.h>
#include <color.h>
#include <display.h>
#include <keyboard.hpp>

void main(void) {
    int oldsec,sec;
    int count = 0;

    /** A counter running at 64Hz from the 128Hz counter
     *
     * This Port is made up of 6 bits
     * |x|1|2|4|8|16|32|64|
     *
     * This port is updated when the 128 Hz timer carries over.
     * Therefore, bit 1 is set every 64 Hz, but stays set.
     * you must wait for the bit to return to 0 if you want to opperate at 64 Hz.
     */
    volatile unsigned char *port64 = 0xA413FEC0;
    /** A status register for the RTC unit.
     *
     * bit 7 is set after a read/write to an RTC register if the 128 Hz timer carries.
     * If this happens, any reads or writes are invalid and must be repeated.
     * Bit 7 must be manually reset to 0.
     *
     * Usage:
     * - reset bit 7 of RCR1
     * - read R64CNT
     * - if bit 7 of RCR1 is set, reset bit 7 and reread R64CNT
     */
    volatile unsigned char *portRCR1 = 0xA413FEDC;
    /** (For all below) Packed digits of 2 digit numbers.
     *
     * Writing is allowed, but must follow a procedure.  See the SH7720 docs for more info.
     * RCR1 bit 7 applies to this for both reads and writes.
     *
     * This register contains a packed 2 digit number in 134 format. Example below
     * |7|6 - 4|3 - 0
     * ----------------
     * |x| 10s |  1s
     *
     * So, bits 0-4 are the first number from 0-9
     * bits 4-6 are the second number from 0-5
     */
    volatile unsigned char *ports = 0xA413FEC2;
    volatile unsigned char *portm = 0xA413FEC4;
    volatile unsigned char *porth = 0xA413FEC6;

    *ports = 0;
    *portm = 0;
    *porth = 0;

    char buffer[10];
    buffer[0] = ' ';
    buffer[1] = ' ';
    Bdisp_EnableColor(1);

    while (1) {
        if(PRGM_GetKey() == 48)
	{
	  int r,c,k;
	  
	  GetKeyWait_OS(&c,&r,0,0,0,&k);
	}
        Bdisp_AllClr_VRAM();
        while (*port64 & 0b00100000);

        while (!(*port64 & 0b00100000));
        count++;

        itoa(count, buffer+2);
        PrintXY(1,1,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);

        itoa(((*porth >> 4) * 10) + (*porth & 0b1111), buffer+2);
        PrintXY(4,2,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);

        buffer[2] = ':';
        buffer[3] = 0;
        PrintXY(6,2,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);
        itoa(((*portm >> 4) * 10) + (*portm & 0b1111), buffer+2);
        PrintXY(7,2,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);

        buffer[2] = ':';
        buffer[3] = 0;
        PrintXY(9,2,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);
        itoa(((*ports >> 4) * 10) + (*ports & 0b1111), buffer+2);
        PrintXY(10,2,buffer, TEXT_MODE_NORMAL, TEXT_COLOR_BLACK);

        Bdisp_PutDisp_DD();

    }
    return;

}